﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;

namespace DarkHttpServer
{
    //Reuquest object
    public class CompactRequest
    {
        public string Method, Url, Protocol;
        public Dictionary<string, string> Headers;
        //Get StreamReader to parse request data
        public CompactRequest(StreamReader sr)
        {
            //First row is formatted as GET /index.html HTTP/1.1
            string firstLine = sr.ReadLine();
            string[] p = firstLine.Split(' ');
            Method = p[0];
            Url = (p.Length > 1) ? p[1] : "NA";
            Protocol = (p.Length > 2) ? p[2] : "NA";
            //Read other request headers
            string line = null;
            Headers = new Dictionary<string, string>();
            while (!string.IsNullOrEmpty(line = sr.ReadLine()))
            {
                int pos = line.IndexOf(":");
                if (pos > -1)
                    Headers.Add(line.Substring(0, pos),
                        line.Substring(pos + 1));
            }
        }
    }
    //Response object
    public class CompactResponse
    {
        //Predefined 200, 404, 500 status
        public class HttpStatus
        {
            public static string Http200 = "200 OK";
            public static string Http404 = "404 Not Found";
            public static string Http500 = "500 Error";
        }
        public string StatusText = HttpStatus.Http200;
        public string ContentType = "text/plain";
        //Supports response headers
        public Dictionary<string, string> Headers
            = new Dictionary<string, string>();
        //The data to response, in byte[]
        public byte[] Data = new byte[] { };
    }
    //Simple but workable HTTP Server
    public class MicroHttpServer
    {
        private Thread serverThread;
        TcpListener listener;
        //Caller have to prepare a callback function to receive CompactRequest
        //and return CompactResponse
        public MicroHttpServer(int port,
            Func<CompactRequest, CompactResponse> reqProc)
        {
            IPAddress ipAddr = IPAddress.Any; //IPAddress.Parse("192.168.1.136");
            listener = new TcpListener(ipAddr, port);
            //Init a new thread
            serverThread = new Thread(() =>
            {
                listener.Start();
                while (true)
                {
                    Socket s = listener.AcceptSocket();
                    try
                    {
                        NetworkStream ns = new NetworkStream(s);
                        //Parse the request
                        StreamReader sr = new StreamReader(ns);
                        CompactRequest req = new CompactRequest(sr);
                        //Callback to get the response
                        CompactResponse resp = reqProc(req);
                        //Return response
                        StreamWriter sw = new StreamWriter(ns);
                        sw.WriteLine("HTTP/1.1 {0}", resp.StatusText);
                        sw.WriteLine("Content-Type: " + resp.ContentType);
                        foreach (string k in resp.Headers.Keys)
                            sw.WriteLine("{0}: {1}", k, resp.Headers[k]);
                        sw.WriteLine("Content-Length: {0}", resp.Data.Length);
                        sw.WriteLine();
                        sw.Flush();
                        //Write the byte array data
                        s.Send(resp.Data);
                        //Close connection
                        s.Shutdown(SocketShutdown.Both);
                        ns.Close();
                    }
                    catch
                    {
                        //Ignore connection errors
                    }
                }
            });
            serverThread.Start();
        }
        public void Stop()
        {
            listener.Stop();
            serverThread.Abort();
        }
    }
}